@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'sprockets_test'
 require 'rack/mock'
 require 'execjs'
@@ -37,28 +38,25 @@ module EnvironmentTests
   test "eco templates" do
     asset = @env["goodbye.js"]
     context = ExecJS.compile(asset.to_s)
-    assert_equal "Goodbye world\n", context.call("JST['goodbye']", :name => "world")
+    assert_equal "Goodbye world\n", context.call("JST['goodbye']", name: "world")
   end
 
   test "ejs templates" do
     assert asset = @env["hello.js"]
     context = ExecJS.compile(asset.to_s)
-    assert_equal "hello: world\n", context.call("JST['hello']", :name => "world")
+    assert_equal "hello: world\n", context.call("JST['hello']", name: "world")
   end
 
-  test "another ejs templates" do
-    assert asset = @env["hello2.js"]
-    context = ExecJS.compile(asset.to_s)
-    assert_equal "hello2: world\n", context.call("JST2['hello2']", :name => "world")
+  test "find_asset! does not raise an exception when asset is found" do
+    @env.find_asset!("hello.js") # assert no raise
   end
 
-  test "angular templates" do
-    assert asset = @env["ng-view.js"]
-    assert_equal <<-JS, asset.to_s
-$app.run(function($templateCache) {
-  $templateCache.put('ng-view.html', "<div ng-view></div>");
-});
-    JS
+  test "find_asset! raises an error when asset is not found" do
+    does_not_exist_file_name = "doesnotexist.blerg"
+    error = assert_raises(Sprockets::NotFound) do
+      @env.find_asset!(does_not_exist_file_name)
+    end
+    assert_match %r{#{does_not_exist_file_name}}, error.message
   end
 
   test "asset_data_uri helper" do
@@ -85,12 +83,12 @@ $app.run(function($templateCache) {
     assert_equal fixture_path('default/gallery.js'), asset.filename
 
     assert asset = @env.find_asset("gallery", accept: 'text/css, application/javascript')
-    assert_equal fixture_path('default/gallery.css.erb'), asset.filename
+    assert_equal fixture_path('default/gallery.js'), asset.filename
 
     assert asset = @env.find_asset("coffee/foo", accept: "application/javascript")
     assert_equal fixture_path('default/coffee/foo.coffee'), asset.filename
 
-    assert asset = @env.find_asset("coffee/foo.coffee", accept: "application/javascript")
+    assert asset = @env.find_asset("coffee/foo.js", accept: "application/javascript")
     assert_equal fixture_path('default/coffee/foo.coffee'), asset.filename
 
     assert asset = @env.find_asset("jquery.tmpl.min", accept: 'application/javascript')
@@ -218,6 +216,51 @@ $app.run(function($templateCache) {
     refute @env.find_asset("zeroclipboard.unknown")
   end
 
+
+  test "explicit package.json access returns json file" do
+    assert_equal fixture_path('default/npm/package.json'),
+      @env["npm/package.json"].filename
+  end
+
+  test "find default npm main" do
+    assert_equal fixture_path('default/npm/main.js'),
+      @env["npm"].filename
+  end
+
+  test "find nested npm main" do
+    assert_equal fixture_path('default/npm-nested/a/b/main.js'),
+      @env["npm-nested/a/b"].filename
+  end
+
+  test "find npm main by format extension" do
+    assert_equal fixture_path('default/npm/main.js'),
+      @env["npm.js"].filename
+  end
+
+  test "find npm main by content type" do
+    assert_equal fixture_path('default/npm/main.js'),
+      @env.find_asset("npm", accept: 'application/javascript').filename
+    assert_equal fixture_path('default/npm/main.js'),
+      @env.find_asset("npm.js", accept: 'application/javascript').filename
+  end
+
+  test "find default npm main when main key is not set" do
+    assert_equal fixture_path('default/express/index.js'),
+      @env["express"].filename
+  end
+
+  test "find npm style by format extension" do
+    assert_equal fixture_path('default/npm/style.css'),
+      @env["npm.css"].filename
+  end
+
+  test "find npm style by content type" do
+    assert_equal fixture_path('default/npm/style.css'),
+      @env.find_asset("npm", accept: 'text/css').filename
+    assert_equal fixture_path('default/npm/style.css'),
+      @env.find_asset("npm.css", accept: 'text/css').filename
+  end
+
   test "find bundled asset in environment" do
     assert_equal "var Gallery = {};\n", @env["gallery.js"].to_s
   end
@@ -244,7 +287,7 @@ $app.run(function($templateCache) {
   end
 
   test "find index.css in directory" do
-    assert_equal ".c {}\n.d {}\n/*\n\n */\n\n", @env["mobile.css"].to_s
+    assert_equal ".c {}\n.d {}\n/*\n\n */\n", @env["mobile.css"].to_s
   end
 
   test "ignore index.min.js in directory" do
@@ -261,32 +304,42 @@ $app.run(function($templateCache) {
   end
 
   test "find erb assets" do
-    assert asset = @env.find_asset("erb/a")
-    assert_equal "text/plain", asset.content_type
-
-    assert asset = @env.find_asset("erb/b")
-    assert_equal "text/plain", asset.content_type
-
-    assert asset = @env.find_asset("erb/c")
-    assert_equal "application/javascript", asset.content_type
-
-    assert asset = @env.find_asset("erb/d")
-    assert_equal "text/css", asset.content_type
-
-    assert asset = @env.find_asset("erb/e")
+    assert asset = @env.find_asset("erb/a.html")
     assert_equal "text/html", asset.content_type
 
-    assert asset = @env.find_asset("erb/f")
+    assert asset = @env.find_asset("erb/b.txt")
+    assert_equal "text/plain", asset.content_type
+
+    assert asset = @env.find_asset("erb/c.js")
+    assert_equal "application/javascript", asset.content_type
+
+    assert asset = @env.find_asset("erb/d.css")
+    assert_equal "text/css", asset.content_type
+
+    assert asset = @env.find_asset("erb/e.html")
+    assert_equal "text/html", asset.content_type
+
+    assert asset = @env.find_asset("erb/f.yaml")
     assert_equal "text/yaml", asset.content_type
   end
 
+  test "es6 asset" do
+    assert asset = @env.find_asset("future.js")
+    assert_match(/var square/, asset.to_s)
+    assert_match(/function/, asset.to_s)
+  end
+
   test "find html builder asset" do
     assert asset = @env.find_asset("nokogiri-html.html")
     assert_equal "text/html", asset.content_type
-    assert_equal <<-HTML, asset.to_s
+    if RUBY_PLATFORM.include?('java')
+      assert_equal '<html><body><span class="bold">Hello world</span></body></html>', asset.to_s
+    else
+      assert_equal <<-HTML, asset.to_s
 <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">
 <html><body><span class="bold">Hello world</span></body></html>
     HTML
+    end
   end
 
   test "find xml builder asset" do
@@ -361,7 +414,7 @@ $app.run(function($templateCache) {
   end
 
   test "missing asset returns nil" do
-    assert_equal nil, @env["missing.js"]
+    assert_nil @env["missing.js"]
   end
 
   test "missing asset path returns nil" do
@@ -409,11 +462,11 @@ $app.run(function($templateCache) {
 
   test "asset logical path for absolute path" do
     assert_equal "gallery.js",
-      @env[fixture_path("default/gallery.js")].logical_path
+      @env.find_asset(fixture_path("default/gallery.js")).logical_path
     assert_equal "application.js",
-      @env[fixture_path("default/application.js.coffee")].logical_path
+      @env.find_asset(fixture_path("default/application.coffee"), accept: "application/javascript").logical_path
     assert_equal "mobile/a.js",
-      @env[fixture_path("default/mobile/a.js")].logical_path
+      @env.find_asset(fixture_path("default/mobile/a.js")).logical_path
   end
 
   test "mobile index logical path shorthand" do
@@ -423,39 +476,6 @@ $app.run(function($templateCache) {
       @env[fixture_path("default/mobile-min/index.min.js")].logical_path
   end
 
-  FIXTURE_ROOT = Sprockets::TestCase::FIXTURE_ROOT
-  FILES_IN_PATH = Dir["#{FIXTURE_ROOT}/default/**/*"].size - 10
-
-  test "iterate over each logical path" do
-    paths = []
-    paths = @env.logical_paths.to_a.map(&:first)
-    assert_equal FILES_IN_PATH, paths.length
-    assert_equal paths.size, paths.uniq.size, "has duplicates"
-
-    assert paths.include?("application.js")
-    assert paths.include?("coffee/foo.js")
-    assert paths.include?("coffee.js")
-    assert !paths.include?("coffee")
-  end
-
-  test "iterate over each logical path and filename" do
-    paths = []
-    filenames = []
-    @env.logical_paths.each do |logical_path, filename|
-      paths << logical_path
-      filenames << filename
-    end
-    assert_equal FILES_IN_PATH, paths.length
-    assert_equal paths.size, paths.uniq.size, "has duplicates"
-
-    assert paths.include?("application.js")
-    assert paths.include?("coffee/foo.js")
-    assert paths.include?("coffee.js")
-    assert !paths.include?("coffee")
-
-    assert filenames.any? { |p| p =~ /application.js.coffee/ }
-  end
-
   test "CoffeeScript files are compiled in a closure" do
     script = @env["coffee"].to_s
     assert_equal "undefined", ExecJS.exec(script)
@@ -484,6 +504,28 @@ $app.run(function($templateCache) {
     assert_equal "text/plain", asset.content_type
     assert_equal "hello.source.txt", asset.logical_path
   end
+
+  test "find source for concatenated asset" do
+    assert asset = @env.find_asset("application.source.coffee")
+    assert_equal "text/coffeescript", asset.content_type
+    assert_equal "application.source.coffee", asset.logical_path
+
+    assert asset = @env.find_asset("application.js")
+    assert_equal "application/javascript", asset.content_type
+    assert_equal "application.js", asset.logical_path
+
+    assert asset = @env.find_asset("application.self.js")
+    assert_equal "application/javascript", asset.content_type
+    assert_equal "application.self.js", asset.logical_path
+
+    assert asset = @env.find_asset("project.self.js")
+    assert_equal "application/javascript", asset.content_type
+    assert_equal "project.self.js", asset.logical_path
+
+    assert asset = @env.find_asset("application.js.map")
+    assert_equal "application/js-sourcemap+json", asset.content_type
+    assert_equal "application.js.map", asset.logical_path
+  end
 end
 
 class WhitespaceProcessor
@@ -526,6 +568,13 @@ class TestEnvironment < Sprockets::TestCase
     assert_equal true, @env.gzip?
   end
 
+  test "change jst template namespace" do
+    @env.register_transformer 'application/javascript+function', 'application/javascript', Sprockets::JstProcessor.new(namespace: 'this.JST2')
+    assert asset = @env["hello.js"]
+    context = ExecJS.compile(asset.to_s)
+    assert_equal "hello: world\n", context.call("JST2['hello']", name: "world")
+  end
+
   test "register bundle processor" do
     old_size = @env.bundle_processors['text/css'].size
     @env.register_bundle_processor 'text/css', WhitespaceProcessor
@@ -538,56 +587,6 @@ class TestEnvironment < Sprockets::TestCase
     assert @env.compressors['text/css'][:whitespace]
   end
 
-  test "register global block preprocessor" do
-    Sprockets::SilenceDeprecation.silence do
-      old_size = new_environment.preprocessors['text/css'].size
-      Sprockets.register_preprocessor('text/css', :foo) { |context, data| data }
-      assert_equal old_size+1, new_environment.preprocessors['text/css'].size
-      Sprockets.unregister_preprocessor('text/css', :foo)
-      assert_equal old_size, new_environment.preprocessors['text/css'].size
-    end
-  end
-
-  test "unregister custom block preprocessor" do
-    Sprockets::SilenceDeprecation.silence do
-      old_size = @env.preprocessors['text/css'].size
-      @env.register_preprocessor('text/css', :foo) { |context, data| data }
-      assert_equal old_size+1, @env.preprocessors['text/css'].size
-      @env.unregister_preprocessor('text/css', :foo)
-      assert_equal old_size, @env.preprocessors['text/css'].size
-    end
-  end
-
-  test "unregister custom block postprocessor" do
-    Sprockets::SilenceDeprecation.silence do
-      old_size = @env.postprocessors['text/css'].size
-      @env.register_postprocessor('text/css', :foo) { |context, data| data }
-      assert_equal old_size+1, @env.postprocessors['text/css'].size
-      @env.unregister_postprocessor('text/css', :foo)
-      assert_equal old_size, @env.postprocessors['text/css'].size
-    end
-  end
-
-  test "register global block postprocessor" do
-    Sprockets::SilenceDeprecation.silence do
-      old_size = new_environment.postprocessors['text/css'].size
-      Sprockets.register_postprocessor('text/css', :foo) { |context, data| data }
-      assert_equal old_size+1, new_environment.postprocessors['text/css'].size
-      Sprockets.unregister_postprocessor('text/css', :foo)
-      assert_equal old_size, new_environment.postprocessors['text/css'].size
-    end
-  end
-
-  test "unregister custom block bundle processor" do
-    Sprockets::SilenceDeprecation.silence do
-      old_size = @env.bundle_processors['text/css'].size
-      @env.register_bundle_processor('text/css', :foo) { |context, data| data }
-      assert_equal old_size+1, @env.bundle_processors['text/css'].size
-      @env.unregister_bundle_processor('text/css', :foo)
-      assert_equal old_size, @env.bundle_processors['text/css'].size
-    end
-  end
-
   test "register global bundle processor" do
     old_size = Sprockets.bundle_processors['text/css'].size
     Sprockets.register_bundle_processor 'text/css', WhitespaceProcessor
@@ -721,10 +720,64 @@ class TestEnvironment < Sprockets::TestCase
     assert_equal 2, asset.metadata[:selector_count]
   end
 
-  test "changing version doesn't affect the assets digest" do
-    old_asset_digest = @env["gallery.js"].hexdigest
+  test "changing version changes the digest_path of the asset" do
+    old_asset_digest = @env["gallery.js"].digest_path
     @env.version = 'v2'
-    assert old_asset_digest == @env["gallery.js"].hexdigest
+    refute_equal old_asset_digest, @env["gallery.js"].digest_path
+  end
+
+  test "changing version changes the digest_path of the asset when there is no preposessor" do
+    old_asset_digest = @env["blank.gif"].digest_path
+    @env.version = 'v2'
+    refute_equal old_asset_digest, @env["blank.gif"].digest_path
+  end
+
+  test "changing version changes the etag of the asset" do
+    old_asset_etag = @env["gallery.js"].etag
+    @env.version = 'v2'
+    new_asset_etag =  @env["gallery.js"].etag
+    refute_equal old_asset_etag, new_asset_etag
+    assert_equal old_asset_etag.size, new_asset_etag.size
+  end
+
+  test "changing version to nil does not break etag" do
+    old_asset_etag = @env["gallery.js"].etag
+    @env.version = nil
+    new_asset_etag =  @env["gallery.js"].etag
+    assert_equal old_asset_etag, new_asset_etag
+    assert_equal old_asset_etag.size, new_asset_etag.size
+  end
+
+  test "changing version does not changes the digest of the asset" do
+    old_asset_digest = @env["gallery.js"].digest
+
+    @env.version = 'v2'
+
+    assert_equal old_asset_digest, @env["gallery.js"].digest
+  end
+
+  test "changing version does not changes the hexdigest of the asset" do
+    old_asset_hexdigest = @env["gallery.js"].hexdigest
+
+    @env.version = 'v2'
+
+    assert_equal old_asset_hexdigest, @env["gallery.js"].hexdigest
+  end
+
+  test "changing version does not changes the base64digest of the asset" do
+    old_asset_base64digest = @env["gallery.js"].base64digest
+
+    @env.version = 'v2'
+
+    assert_equal old_asset_base64digest, @env["gallery.js"].base64digest
+  end
+
+  test "changing version does not changes the integrity of the asset" do
+    old_asset_integrity = @env["gallery.js"].integrity
+
+    @env.version = 'v2'
+
+    assert_equal old_asset_integrity, @env["gallery.js"].integrity
   end
 
   test "bundled asset is stale if its mtime is updated or deleted" do
@@ -803,14 +856,29 @@ class TestEnvironment < Sprockets::TestCase
   end
 
   test "disabling default directive preprocessor" do
-    assert processor = @env.preprocessors['application/javascript'][0]
-    assert_kind_of Sprockets::DirectiveProcessor, processor
+    assert processors = @env.preprocessors['application/javascript']
+    processor = processors.detect {|p| p.is_a?(Sprockets::DirectiveProcessor) }
+
+    assert processor
     @env.unregister_preprocessor('application/javascript', processor)
-    assert_equal "// =require \"notfound\"\n;\n", @env["missing_require.js"].to_s
+    assert_equal "// =require \"notfound\";\n", @env["missing_require.js"].to_s
+  end
+
+  test "disabling processors by class name also disables processors which are instances of that class" do
+    space_and_whitespace_compressor = Class.new(WhitespaceCompressor)
+
+    @env.register_preprocessor('text/html', WhitespaceCompressor)
+    @env.register_preprocessor('text/html', WhitespaceCompressor.new)
+    @env.register_preprocessor('text/html', space_and_whitespace_compressor)
+    assert_equal 3, @env.preprocessors['text/html'].size
+
+    @env.unregister_preprocessor('text/html', WhitespaceCompressor)
+    assert_equal 1, @env.preprocessors['text/html'].size
+    assert_equal space_and_whitespace_compressor, @env.preprocessors['text/html'][0]
   end
 end
 
-class TestCached < Sprockets::TestCase
+class TestCachedEnvironment < Sprockets::TestCase
   include EnvironmentTests
 
   def new_environment
@@ -838,7 +906,7 @@ class TestCached < Sprockets::TestCase
 
   test "does not allow new mime types to be added" do
     assert_raises RuntimeError do
-      @env.register_mime_type "application/javascript", ".jst"
+      @env.register_mime_type "application/javascript", extensions: [".jst"]
     end
   end
 
