@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'sprockets_test'
 require 'fileutils'
 require 'tmpdir'
@@ -8,15 +9,34 @@ class TestManifest < Sprockets::TestCase
     @env = Sprockets::Environment.new(".") do |env|
       env.append_path(fixture_path('default'))
     end
-    @dir = File.join(Dir::tmpdir, 'sprockets/manifest')
+
+    @dir = Dir.mktmpdir
     FileUtils.mkdir_p(@dir)
   end
 
   def teardown
-    # FileUtils.rm_rf(@dir)
-    # wtf, dunno
-    system "rm -rf #{@dir}"
-    assert Dir["#{@dir}/*"].empty?
+    FileUtils.remove_entry_secure(@dir) if File.exist?(@dir)
+    assert Dir["#{@dir}/*"].empty?, "Expected #{Dir["#{@dir}/*"]} to be empty but it was not"
+  end
+
+  test 'linking same file twice does not raise an error' do
+    config_manifest = 'link_same_file_twice.js'
+
+    @env.append_path(fixture_path('double'))
+    output_manifest_json = File.join(@dir, "manifest.json")
+    manifest = Sprockets::Manifest.new(@env, @dir, output_manifest_json)
+    manifest.compile(config_manifest)
+  end
+
+  test 'double rendering with link_directory raises an error' do
+    config_manifest = 'link_directory_manifest.js'
+
+    @env.append_path(fixture_path('double'))
+    output_manifest_json = File.join(@dir, "manifest.json")
+    manifest = Sprockets::Manifest.new(@env, @dir, output_manifest_json)
+    assert_raises(Sprockets::DoubleLinkError) do
+      manifest.compile(config_manifest)
+    end
   end
 
   test "specify full manifest filename" do
@@ -40,34 +60,6 @@ class TestManifest < Sprockets::TestCase
     assert_match(/^\.sprockets-manifest-[a-f0-9]{32}.json/, File.basename(manifest.filename))
   end
 
-  test "specify manifest directory with existing legacy manifest-abc213.json" do
-    path = File.join(@dir, "manifest-#{SecureRandom.hex(16)}.json")
-
-    File.open(path, 'w') { |f| f.write "{}" }
-
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    assert_equal @dir, manifest.directory
-    assert_match(/^manifest-[a-f0-9]{32}.json/, File.basename(manifest.filename))
-
-    manifest.save
-    assert_match(/^\.sprockets-manifest-[a-f0-9]{32}.json/, File.basename(manifest.filename))
-  end
-
-  test "specify manifest directory with existing legacy manifest.json" do
-    path = File.join(@dir, "manifest.json")
-
-    File.open(path, 'w') { |f| f.write "{}" }
-
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    assert_equal @dir, manifest.directory
-    assert_equal "manifest.json", File.basename(manifest.filename)
-
-    manifest.save
-    assert_match(/^\.sprockets-manifest-[a-f0-9]{32}.json/, File.basename(manifest.filename))
-  end
-
   test "specify manifest directory with existing .sprockets-manifest-*.json" do
     path = File.join(@dir, ".sprockets-manifest-#{SecureRandom.hex(16)}.json")
     File.open(path, 'w') { |f| f.write "{}" }
@@ -94,6 +86,7 @@ class TestManifest < Sprockets::TestCase
   end
 
   test "compile asset" do
+    @env.version = '1.1.2'
     manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
 
     digest_path = @env['application.js'].digest_path
@@ -114,6 +107,71 @@ class TestManifest < Sprockets::TestCase
     assert_equal digest_path, data['assets']['application.js']
   end
 
+  test "compile index asset" do
+    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
+
+    digest_path = @env['coffee/index.js'].digest_path
+    assert_match(/coffee\/index-\w+.js/, digest_path)
+
+    assert !File.exist?("#{@dir}/#{digest_path}")
+
+    manifest.compile('coffee/index.js')
+
+    assert File.exist?("#{@dir}/#{digest_path}")
+
+    data = JSON.parse(File.read(manifest.filename))
+    assert data['files'][digest_path]
+    assert_equal "coffee/index.js", data['files'][digest_path]['logical_path']
+    assert_equal digest_path, data['assets']['coffee/index.js']
+  end
+
+  test "compile index asset by alias" do
+    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
+
+    digest_path = @env['coffee.js'].digest_path
+    assert_match(/coffee-\w+.js/, digest_path)
+
+    assert !File.exist?("#{@dir}/#{digest_path}")
+
+    manifest.compile('coffee.js')
+
+    assert File.exist?("#{@dir}/#{digest_path}")
+
+    data = JSON.parse(File.read(manifest.filename))
+    assert data['files'][digest_path]
+    assert_equal "coffee.js", data['files'][digest_path]['logical_path']
+    assert !data['assets']['coffee/index.js']
+    assert_equal digest_path, data['assets']['coffee.js']
+  end
+
+
+  test "compile asset with aliased index links" do
+    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
+
+    main_digest_path = @env['alias-index-link.js'].digest_path
+    dep_digest_path = @env['coffee.js'].digest_path
+
+    assert !File.exist?("#{@dir}/#{main_digest_path}")
+    assert !File.exist?("#{@dir}/#{dep_digest_path}")
+
+    manifest.compile('alias-index-link.js')
+    assert File.directory?(manifest.directory)
+    assert File.file?(manifest.filename)
+
+    assert File.exist?("#{@dir}/manifest.json")
+    assert File.exist?("#{@dir}/#{main_digest_path}")
+    assert File.exist?("#{@dir}/#{dep_digest_path}")
+
+    data = JSON.parse(File.read(manifest.filename))
+
+    assert data['files'][main_digest_path]
+    assert data['files'][dep_digest_path]
+    assert_equal "alias-index-link.js", data['files'][main_digest_path]['logical_path']
+    assert_equal "coffee.js", data['files'][dep_digest_path]['logical_path']
+    assert_equal main_digest_path, data['assets']['alias-index-link.js']
+    assert_equal dep_digest_path, data['assets']['coffee.js']
+  end
+
   test "compile to directory and seperate location" do
     manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
 
@@ -155,18 +213,18 @@ class TestManifest < Sprockets::TestCase
   test "compile asset with absolute path" do
     manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
 
-    digest_path = @env['application.js'].digest_path
+    digest_path = @env['gallery.js'].digest_path
 
     assert !File.exist?("#{@dir}/#{digest_path}")
 
-    manifest.compile(fixture_path('default/application.js.coffee'))
+    manifest.compile(fixture_path('default/gallery.js'))
 
     assert File.exist?("#{@dir}/manifest.json")
     assert File.exist?("#{@dir}/#{digest_path}")
 
     data = JSON.parse(File.read(manifest.filename))
     assert data['files'][digest_path]
-    assert_equal digest_path, data['assets']['application.js']
+    assert_equal digest_path, data['assets']['gallery.js']
   end
 
   test "compile multiple assets" do
@@ -192,14 +250,13 @@ class TestManifest < Sprockets::TestCase
   end
 
   test "compile with transformed asset" do
-    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
-
     assert svg_digest_path = @env['logo.svg'].digest_path
     assert png_digest_path = @env['logo.png'].digest_path
 
     assert !File.exist?("#{@dir}/#{svg_digest_path}")
     assert !File.exist?("#{@dir}/#{png_digest_path}")
 
+    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
     manifest.compile('logo.svg', 'logo.png')
 
     assert File.exist?("#{@dir}/manifest.json")
@@ -271,52 +328,34 @@ class TestManifest < Sprockets::TestCase
     assert_equal subdep_digest_path, data['assets']['gallery.js']
   end
 
-  test "compile index asset" do
+  test "recompile asset when environment version is changed" do
     manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
 
-    digest_path = @env['coffee.js'].digest_path
-    assert_match(/coffee\/index-\w+.js/, digest_path)
+    digest_path = @env['application.js'].digest_path
+    filename = fixture_path('default/application.coffee')
 
-    assert !File.exist?("#{@dir}/#{digest_path}")
+    sandbox filename do
+      assert !File.exist?("#{@dir}/#{digest_path}"), Dir["#{@dir}/*"].inspect
 
-    manifest.compile('coffee.js')
+      manifest.compile('application.js')
 
-    assert File.exist?("#{@dir}/#{digest_path}")
+      assert File.exist?("#{@dir}/manifest.json")
+      assert File.exist?("#{@dir}/#{digest_path}")
 
-    data = JSON.parse(File.read(manifest.filename))
-    assert data['files'][digest_path]
-    assert_equal "coffee/index.js", data['files'][digest_path]['logical_path']
-    assert_equal digest_path, data['assets']['coffee/index.js']
-    assert_equal digest_path, data['assets']['coffee.js']
-  end
+      @env.version = '1.1.3'
+      new_digest_path = @env['application.js'].digest_path
 
-  test "compile with regex" do
-    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
-
-    app_digest_path = @env['application.js'].digest_path
-    gallery_digest_path = @env['gallery.css'].digest_path
-
-    assert !File.exist?("#{@dir}/#{app_digest_path}")
-    assert !File.exist?("#{@dir}/#{gallery_digest_path}")
-
-    manifest.compile('gallery.css', /application.js/)
-
-    assert File.exist?("#{@dir}/manifest.json")
-    assert File.exist?("#{@dir}/#{app_digest_path}")
-    assert File.exist?("#{@dir}/#{gallery_digest_path}")
-
-    data = JSON.parse(File.read(manifest.filename))
-    assert data['files'][app_digest_path]
-    assert data['files'][gallery_digest_path]
-    assert_equal app_digest_path, data['assets']['application.js']
-    assert_equal gallery_digest_path, data['assets']['gallery.css']
+      assert !File.exist?("#{@dir}/#{new_digest_path}"), Dir["#{@dir}/*"].inspect
+      manifest.compile('application.js')
+      assert File.exist?("#{@dir}/#{new_digest_path}")
+    end
   end
 
   test "recompile asset" do
     manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
 
     digest_path = @env['application.js'].digest_path
-    filename = fixture_path('default/application.js.coffee')
+    filename = fixture_path('default/application.coffee')
 
     sandbox filename do
       assert !File.exist?("#{@dir}/#{digest_path}"), Dir["#{@dir}/*"].inspect
@@ -373,7 +412,7 @@ class TestManifest < Sprockets::TestCase
     manifest = Sprockets::Manifest.new(@env, @dir)
 
     digest_path = @env['application.js'].digest_path
-    filename = fixture_path('default/application.js.coffee')
+    filename = fixture_path('default/application.coffee')
 
     sandbox filename do
       manifest.compile('application.js')
@@ -401,30 +440,33 @@ class TestManifest < Sprockets::TestCase
     manifest = Sprockets::Manifest.new(@env, @dir)
 
     digest_path = @env['application.js'].digest_path
-    filename = fixture_path('default/application.js.coffee')
+    filename = fixture_path('default/application.coffee')
 
     sandbox filename do
       manifest.compile('application.js')
       assert File.exist?("#{@dir}/#{digest_path}")
 
+      Timecop.travel(Time.now + 1)
       File.open(filename, 'w') { |f| f.write "a;" }
-      mtime = Time.now + 1
+      mtime = Time.now
       File.utime(mtime, mtime, filename)
       new_digest_path1 = @env['application.js'].digest_path
 
       manifest.compile('application.js')
       assert File.exist?("#{@dir}/#{new_digest_path1}")
 
+      Timecop.travel(Time.now + 1)
       File.open(filename, 'w') { |f| f.write "b;" }
-      mtime = Time.now + 2
+      mtime = Time.now
       File.utime(mtime, mtime, filename)
       new_digest_path2 = @env['application.js'].digest_path
 
       manifest.compile('application.js')
       assert File.exist?("#{@dir}/#{new_digest_path2}")
 
+      Timecop.travel(Time.now + 1)
       File.open(filename, 'w') { |f| f.write "c;" }
-      mtime = Time.now + 3
+      mtime = Time.now
       File.utime(mtime, mtime, filename)
       new_digest_path3 = @env['application.js'].digest_path
 
@@ -445,53 +487,8 @@ class TestManifest < Sprockets::TestCase
       assert data['files'][new_digest_path3]
       assert_equal new_digest_path3, data['assets']['application.js']
     end
-  end
-
-  test "remove old backups(age)" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    filename = fixture_path('default/application.js.coffee')
-
-    sandbox filename do
-
-      File.open(filename, 'w') { |f| f.write "a;" }
-      mtime = Time.local(2014, 4, 26, 12, 0, 0)
-      File.utime(mtime, mtime, filename)
-      new_digest_path1 = @env['application.js'].digest_path
-
-      manifest.compile('application.js')
-      assert File.exist?("#{@dir}/#{new_digest_path1}")
-
-      File.open(filename, 'w') { |f| f.write "b;" }
-      mtime = Time.local(2014, 4, 27, 8, 0, 0)
-      File.utime(mtime, mtime, filename)
-      new_digest_path2 = @env['application.js'].digest_path
-
-      manifest.compile('application.js')
-      assert File.exist?("#{@dir}/#{new_digest_path2}")
-
-      File.open(filename, 'w') { |f| f.write "c;" }
-      mtime = Time.local(2014, 4, 27, 9, 0, 0)
-      File.utime(mtime, mtime, filename)
-      new_digest_path3 = @env['application.js'].digest_path
-
-      manifest.compile('application.js')
-      assert File.exist?("#{@dir}/#{new_digest_path3}")
-
-      Time.stub(:now, Time.local(2014, 4, 27, 12, 0, 0)) do
-        manifest.clean(0, 60 * 60 * 24)
-      end
-
-      assert !File.exist?("#{@dir}/#{new_digest_path1}")
-      assert File.exist?("#{@dir}/#{new_digest_path2}")
-      assert File.exist?("#{@dir}/#{new_digest_path3}")
-
-      data = JSON.parse(File.read(manifest.filename))
-      assert !data['files'][new_digest_path1]
-      assert data['files'][new_digest_path2]
-      assert data['files'][new_digest_path3]
-      assert_equal new_digest_path3, data['assets']['application.js']
-    end
+  ensure
+    Timecop.return
   end
 
   test "test manifest does not exist" do
@@ -553,83 +550,6 @@ class TestManifest < Sprockets::TestCase
     end
   end
 
-  test "find all filenames matching fnmatch filters" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    paths = []
-    manifest.find("mobile/*.js").each do |asset|
-      paths << asset.logical_path
-    end
-
-    assert paths.include?("mobile/a.js")
-    assert paths.include?("mobile/b.js")
-    assert !paths.include?("mobile/c.css")
-  end
-
-  test "iterate over each logical path matches index files" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    paths = []
-    manifest.find("coffee.js").each do |asset|
-      paths << asset.logical_path
-    end
-    assert paths.include?("coffee/index.js")
-    assert !paths.include?("coffee.js")
-  end
-
-  test "each logical path enumerator matching fnmatch filters" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    paths = []
-    enum = manifest.find("mobile/*.js")
-    enum.to_a.each do |asset|
-      paths << asset.logical_path
-    end
-
-    assert paths.include?("mobile/a.js")
-    assert paths.include?("mobile/b.js")
-    assert !paths.include?("mobile/c.css")
-  end
-
-  test "iterate over each logical path matching regexp filters" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    paths = []
-    manifest.find(/mobile\/.*\.js/).each do |asset|
-      paths << asset.logical_path
-    end
-
-    assert paths.include?("mobile/a.js")
-    assert paths.include?("mobile/b.js")
-    assert !paths.include?("mobile/c.css")
-  end
-
-  test "iterate over each logical path matching proc filters" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    paths = []
-    manifest.find(proc { |fn| fn.match("mobile/") && File.extname(fn) == '.js' }).each do |asset|
-      paths << asset.logical_path
-    end
-
-    assert paths.include?("mobile/a.js")
-    assert paths.include?("mobile/b.js")
-    assert !paths.include?("mobile/c.css")
-  end
-
-  test "iterate over each logical path matching proc filters with full path arg" do
-    manifest = Sprockets::Manifest.new(@env, @dir)
-
-    paths = []
-    manifest.find(proc { |_, fn| fn.match(fixture_path('default/mobile')) }).each do |asset|
-      paths << asset.logical_path
-    end
-
-    assert paths.include?("mobile/a.js")
-    assert paths.include?("mobile/b.js")
-    assert !paths.include?("application.js")
-  end
-
   test "find_sources with environment" do
     manifest = Sprockets::Manifest.new(@env, @dir)
 
@@ -665,7 +585,6 @@ class TestManifest < Sprockets::TestCase
       original_path = @env[file_name].digest_path
       manifest.compile(file_name)
       assert File.exist?("#{@dir}/#{original_path}.gz"), "Expecting '#{original_path}' to generate gzipped file: '#{original_path}.gz' but it did not"
-      assert_equal File.stat("#{@dir}/#{original_path}").mtime, Zlib::GzipReader.open("#{@dir}/#{original_path}.gz") {|gz| gz.mtime }
     end
   end
 
@@ -684,7 +603,6 @@ class TestManifest < Sprockets::TestCase
       original_path = @env[file_name].digest_path
       manifest.compile(file_name)
       assert File.exist?("#{@dir}/#{original_path}.gz"), "Expecting '#{original_path}' to generate gzipped file: '#{original_path}.gz' but it did not"
-      assert_equal File.stat("#{@dir}/#{original_path}").mtime, Zlib::GzipReader.open("#{@dir}/#{original_path}.gz") {|gz| gz.mtime }
     end
   end
 
@@ -709,8 +627,86 @@ class TestManifest < Sprockets::TestCase
 
   test 'raises exception when gzip fails' do
     manifest = Sprockets::Manifest.new(@env, @dir)
-    Zlib::GzipWriter.stub(:new, -> { fail 'kaboom' }) do
-      assert_raises('kaboom') { manifest.compile('application.js') }
+    Zlib::GzipWriter.stub(:new, -> (io, level) { fail 'kaboom' }) do
+      ex = assert_raises(RuntimeError) { manifest.compile('application.js') }
+      assert_equal 'kaboom', ex.message
     end
   end
+
+  # Sleep duration to context switch between concurrent threads.
+  CONTEXT_SWITCH_DURATION = 0.1
+
+  # Record Exporter sequence with a delay to test concurrency.
+  class SlowExporter < Sprockets::Exporters::Base
+    class << self
+      attr_accessor :seq
+    end
+
+    def call
+      SlowExporter.seq << '0'
+      sleep CONTEXT_SWITCH_DURATION
+      SlowExporter.seq << '1'
+    end
+  end
+
+  class SlowExporter2 < SlowExporter
+  end
+
+  test 'concurrent exporting' do
+    # Register 2 exporters and compile 2 files to ensure that
+    # all 4 exporting tasks run concurrently.
+    SlowExporter.seq = []
+    @env.register_exporter 'image/png',SlowExporter
+    @env.register_exporter 'image/png',SlowExporter2
+    Sprockets::Manifest.new(@env, @dir).compile('logo.png', 'troll.png')
+    refute_equal %w(0 1 0 1 0 1 0 1), SlowExporter.seq
+  end
+
+  test 'sequential exporting' do
+    @env.export_concurrent = false
+    SlowExporter.seq = []
+    @env.register_exporter 'image/png',SlowExporter
+    @env.register_exporter 'image/png',SlowExporter2
+    Sprockets::Manifest.new(@env, @dir).compile('logo.png', 'troll.png')
+    assert_equal %w(0 1 0 1 0 1 0 1), SlowExporter.seq
+  end
+
+  # Record Processor sequence with a delay to test concurrency.
+  class SlowProcessor
+    attr_reader :seq
+
+    def initialize
+      @seq = []
+    end
+
+    def call(_)
+      seq << '0'
+      sleep CONTEXT_SWITCH_DURATION
+      seq << '1'
+      nil
+    end
+  end
+
+  test 'concurrent processing' do
+    processor = SlowProcessor.new
+    @env.register_postprocessor 'image/png', processor
+    Sprockets::Manifest.new(@env, @dir).compile('logo.png', 'troll.png')
+    refute_equal %w(0 1 0 1), processor.seq
+  end
+
+  test 'sequential processing' do
+    @env.export_concurrent = false
+    processor = SlowProcessor.new
+    @env.register_postprocessor 'image/png', processor
+    Sprockets::Manifest.new(@env, @dir).compile('logo.png', 'troll.png')
+    assert_equal %w(0 1 0 1), processor.seq
+  end
+
+  test 'clobber works when cache_dir not created' do
+    @cache_dir = File.join(Dir::tmpdir, 'sprockets')
+    refute File.exist?(@cache_dir)
+    @env.cache = Sprockets::Cache::FileStore.new(@cache_dir)
+    manifest = Sprockets::Manifest.new(@env, File.join(@dir, 'manifest.json'))
+    manifest.clobber
+  end
 end
